@@ -66,14 +66,16 @@
 #define STRINGZILLA_H_
 
 #define STRINGZILLA_H_VERSION_MAJOR 4
-#define STRINGZILLA_H_VERSION_MINOR 2
-#define STRINGZILLA_H_VERSION_PATCH 3
+#define STRINGZILLA_H_VERSION_MINOR 4
+#define STRINGZILLA_H_VERSION_PATCH 1
 
 #include "types.h"        // `sz_size_t`, `sz_bool_t`, `sz_ordering_t`
 #include "compare.h"      // `sz_equal`, `sz_order`
 #include "memory.h"       // `sz_copy`, `sz_move`, `sz_fill`
 #include "hash.h"         // `sz_bytesum`, `sz_hash`, `sz_state_init`, `sz_state_stream`, `sz_state_fold`
 #include "find.h"         // `sz_find`, `sz_find_byteset`, `sz_rfind`
+#include "utf8.h"         // `sz_utf8_find_newline`, `sz_utf8_find_whitespace`, `sz_utf8_find_nth`, `sz_utf8_valid`
+#include "utf8_case.h"    // `sz_utf8_case_insensitive_find`, `sz_utf8_unpack_chunk`
 #include "small_string.h" // `sz_string_t`, `sz_string_init`, `sz_string_free`
 #include "sort.h"         // `sz_sequence_argsort`, `sz_pgrams_sort`
 #include "intersect.h"    // `sz_sequence_intersect`
